
function Stretcher (element, stretchDuration, onFinished, isVertical) {

	this.element = element;

	this.startTime = 0;
	this.timer = null;
	
	this.duration = stretchDuration;
	this.multiplier = 1;
	this.stretchTime = 0;
			
	this.onFinished = onFinished;
}

 
Stretcher.prototype.stretch = function (event, sizeTo, widgetReSize, isVertical) {
		
	this.positionTo = sizeTo;	
		
	if (isVertical) {
		this.positionNow = parseInt(document.defaultView.getComputedStyle(this.element, "").getPropertyValue("height"));
	} else {
		this.positionNow = parseInt(document.defaultView.getComputedStyle(this.element, "").getPropertyValue("width"));
	}
	
	if (this.positionTo < this.positionNow) { // is going to shrink
		
		this.minPosition = this.positionTo;
		this.maxPosition = this.positionNow;
	
	} else { // is going to grow
		
		this.minPosition = this.positionNow;
		this.maxPosition = this.positionTo;
	
	}

	if (this.timer != null) return false;

	// slo-mo
	if (event && event != undefined && event.shiftKey) {
		this.multiplier = 10;
	} else {
		this.multiplier = 1;
	}
	
	var timeNow = (new Date).getTime();
	
	if (this.timer != null) {
		clearInterval(this.timer);
		this.timer = null;
		this.stretchTime -= (timeNow - this.startTime);
		this.positionFrom = this.positionNow;
	} else {
		this.stretchTime = this.duration * this.multiplier;
		this.positionFrom = this.positionNow;
	}

	// If we're expanding, resize the window to make room for the newly-sized content
	// We don't want to do this before shrinking because we'll clip the content

	if (window.widget && (this.positionTo == this.maxPosition) && widgetReSize) {
		if (isVertical) {
			window.resizeTo(parseInt(document.defaultView.getComputedStyle(this.element, "").getPropertyValue("width")), this.positionTo);
		} else {
			window.resizeTo(this.positionTo, parseInt(document.defaultView.getComputedStyle(this.element, "").getPropertyValue("height")));
		}
	}

	this.startTime = timeNow - 13; // set it back one frame.
		
	// We need to store this in a local variable so the timer
	// does not lose scope when invoking tick
	
	var localThis = this;
	this.tick(widgetReSize, isVertical);
	this.timer = setInterval (function() { localThis.tick(widgetReSize, isVertical); }, 13);

}
		

// Tick does all the incremental resize work
 
Stretcher.prototype.tick = function (widgetReSize, isVertical) {

	var T;
	var ease;
	var time  = (new Date).getTime();
	var frame;
			
	T = limit_3(time-this.startTime, 0, this.stretchTime);
	ease = 0.5 - (0.5 * Math.cos(Math.PI * T / this.stretchTime));

	if (T >= this.stretchTime) {
		// go to the finished position when the timer is up
		this.positionNow = this.positionTo;
		clearInterval (this.timer);
		this.timer = null;
		
		// If we're shrinking, we resize the window AFTER the animation is complete
		// Otherwise we'll clip the content as it shrinks
		
		if (window.widget && (this.positionTo == this.minPosition) && widgetReSize) {
			if (isVertical) {
				window.resizeTo(parseInt(document.defaultView.getComputedStyle(this.element, "").getPropertyValue("width")), this.positionNow);
			} else {
				window.resizeTo(this.positionNow, parseInt(document.defaultView.getComputedStyle(this.element, "").getPropertyValue("height")));
			}
		}
		if (this.onFinished) {
			// call after the last frame is drawn
			var localThis = this;
			setTimeout (function() { localThis.onFinished(); }, 0);
		}
	} else {
		this.positionNow = parseInt(computeNextFloat(this.positionFrom, this.positionTo, ease));
	}

	if (isVertical) {
		this.element.style.height = this.positionNow + "px";
	} else {
		this.element.style.width = this.positionNow + "px";
	}
	
	
	// A little touch of code to prevent some unsightly overlapping that 
	// was occuring as the widget opened and closed vertically
	
	if (widgetReSize && isVertical) {
	
		if (this.positionNow >= 156) {
		
			document.getElementById('dark_bottom').style.clip = "auto";
			document.getElementById('big_count').style.clip = "auto";
			
		} else  { 
		
			var tmp = 156 - this.positionNow;
			
			document.getElementById('dark_bottom').style.clip = "rect(" + tmp + "px auto auto auto)";
			document.getElementById('big_count').style.clip = "rect(" + tmp + "px auto auto auto)";
		}

	}
}
 
Stretcher.prototype.isStretched = function() {
	return (this.positionNow == this.maxPosition);
}


// APPLE ANIMATION CODE


var animation = {duration:0, starttime:0, to:0, now:0, from:0, firstElement:null, secondElement:null, timer:null};


function animate() {

    var T;
    var ease;
    var time = (new Date).getTime();

    T = limit_3(time - animation.starttime, 0, animation.duration);

    if (T >= animation.duration) {
		clearInterval (animation.timer);
		animation.timer = null;
		animation.now = animation.to;
    } else {
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
		animation.now = computeNextFloat(animation.from, animation.to, ease);
    }

    animation.firstElement.style.opacity = animation.now;
    animation.secondElement.style.opacity = animation.now;
}

function limit_3 (a, b, c) {
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
    return from + (to - from) * ease;
}
